@@ -1,4 +1,4 @@
-# Copyright 2021 VMware, Inc.
+# Copyright 2023-2024 Broadcom
 # SPDX-License-Identifier: Apache-2.0
 import collections
 import logging
@@ -14,6 +14,8 @@ from typing import Tuple
 
 from vdk.internal.builtin_plugins.ingestion.ingester_base import IIngesterPlugin
 from vdk.internal.core import errors
+from vdk.internal.core.errors import ResolvableBy
+from vdk.internal.core.errors import UserCodeError
 from vdk.plugin.sqlite.sqlite_configuration import SQLiteConfiguration
 from vdk.plugin.sqlite.sqlite_connection import SQLiteConnection
 
@@ -55,16 +57,14 @@ class IngestToSQLite(IIngesterPlugin):
         """
         target = target or self.conf.get_sqlite_file()
         if not target:
-            errors.log_and_throw(
-                errors.ResolvableBy.USER_ERROR,
-                log,
-                "Failed to proceed with ingestion",
-                "Target was not supplied as a parameter",
-                "Will not proceed with ingestion",
-                (
+            errors.report_and_throw(
+                UserCodeError(
+                    "Failed to proceed with ingestion",
+                    "Target was not supplied as a parameter",
+                    "Will not proceed with ingestion",
                     "Set target either through the target parameter in send_object_for_ingestion,"
-                    "or through either of the VDK_INGEST_TARGET_DEFAULT or VDK_SQLITE_FILE environment variables"
-                ),
+                    "or through either of the VDK_INGEST_TARGET_DEFAULT or VDK_SQLITE_FILE environment variables",
+                )
             )
         if not payload:
             log.debug(
@@ -89,68 +89,34 @@ class IngestToSQLite(IIngesterPlugin):
     def __ingest_payload(
         self, destination_table: str, payload: List[dict], cur: Cursor
     ) -> None:
-        fields, query = self.__create_query(destination_table, cur)
-
-        for obj in payload:
+        values, query = self.__create_query(destination_table, payload, cur)
+        for obj in values:
             try:
                 cur.execute(query, obj)
                 log.debug("Payload was ingested.")
             except Exception as e:
-                if collections.Counter(fields) != collections.Counter(obj.keys()):
-                    errors.log_and_rethrow(
-                        errors.ResolvableBy.USER_ERROR,
-                        log,
-                        "Failed to sent payload",
-                        f"""
-                        One or more column names in the input data did NOT
-                        match corresponding column names in the database.
-                           Input Table Columns: {list(obj.keys())}
-                        Database Table Columns: {fields}
-                        """,
-                        "Will not be able to send the payload for ingestion",
-                        "See error message for help ",
-                        e,
-                        wrap_in_vdk_error=True,
-                    )
-                elif isinstance(e, ProgrammingError):
-                    errors.log_and_rethrow(
-                        errors.ResolvableBy.USER_ERROR,
-                        log,
-                        "Failed to sent payload",
-                        f"""
-                        An issue with the SQL query occured. The error message
-                        was: {str(e)}
-                        """,
-                        "Will not be able to send the payload for ingestion",
-                        "See error message for help ",
-                        e,
-                        wrap_in_vdk_error=True,
+                if isinstance(e, ProgrammingError):
+                    log.warning(
+                        "Failed to sent payload. An issue with the SQL query occurred."
                     )
+                    errors.report(ResolvableBy.USER_ERROR, e)
                 else:
-                    errors.log_and_rethrow(
-                        errors.ResolvableBy.PLATFORM_ERROR,
-                        log,
-                        "Failed to sent payload",
-                        "Unknown error. Error message was : " + str(e),
-                        "Will not be able to send the payload for ingestion",
-                        "See error message for help ",
-                        e,
-                        wrap_in_vdk_error=True,
-                    )
+                    errors.report(errors.ResolvableBy.PLATFORM_ERROR, e)
+                raise e
 
     def __check_destination_table_exists(
         self, destination_table: str, cur: Cursor
     ) -> None:
         columns = self.__table_columns(cur, destination_table)
         if not columns:  # check table with no columns does not exists
-            errors.log_and_throw(
-                errors.ResolvableBy.USER_ERROR,
-                log,
-                what_happened="Cannot send payload for ingestion to SQLite database.",
-                why_it_happened="destination_table does not exist in the target database.",
-                consequences="Will not be able to send the payloads and will throw exception."
-                "Likely the job would fail",
-                countermeasures="Make sure the destination_table exists in the target SQLite database.",
+            errors.report_and_throw(
+                UserCodeError(
+                    "Cannot send payload for ingestion to SQLite database.",
+                    "destination_table does not exist in the target database.",
+                    "Will not be able to send the payloads and will throw exception."
+                    "Likely the job would fail",
+                    "Make sure the destination_table exists in the target SQLite database.",
+                )
             )
 
     def __table_columns(
@@ -169,18 +135,39 @@ class IngestToSQLite(IIngesterPlugin):
             columns.append((row[0], row[1]))
         return columns
 
-    def __create_query(self, destination_table: str, cur: Cursor) -> Tuple[list, str]:
+    def __create_query(
+        self, destination_table: str, payload: List[dict], cur: Cursor
+    ) -> Tuple[list, str]:
         fields = [
             field_tuple[0]
             for field_tuple in cur.execute(
                 f"SELECT name FROM PRAGMA_TABLE_INFO('{destination_table}')"
             ).fetchall()
         ]
-        # the query fstring evaluates to 'INSERT INTO dest_table (val1, val2, val3) VALUES (:val1, :val2, :val3)'
-        # assuming dest_table is the destination_table and val1, val2, val3 are the fields of that table
-        query = f"INSERT INTO {destination_table} ({', '.join(fields)}) VALUES ({', '.join([':' + field for field in fields])})"
 
-        return fields, query
+        # verify that the payload header and table column names match
+        for obj in payload:
+            if collections.Counter(fields) != collections.Counter(obj.keys()):
+                errors.report_and_throw(
+                    UserCodeError(
+                        "Failed to sent payload",
+                        f"""
+                    One or more column names in the input data did NOT
+                    match corresponding column names in the database.
+                       Input Table Columns: {list(obj.keys())}
+                    Database Table Columns: {fields}
+                    """,
+                        "Will not be able to send the payload for ingestion",
+                        "See error message for help ",
+                    )
+                )
+
+        # the query fstring evaluates to 'INSERT INTO dest_table (val1, val2, val3) VALUES (?, ?, ?)'
+        # assuming dest_table is the destination_table and val1, val2, val3 are the fields of that table
+        values = [[obj.get(field) for field in fields] for obj in payload]
+        fields = [field if " " not in field else f'"{field}"' for field in fields]
+        query = f"INSERT INTO {destination_table} ({', '.join(fields)}) VALUES ({', '.join(['?' for _ in fields])})"
+        return values, query
 
     def __create_table_if_not_exists(
         self, cur: Cursor, destination_table: str, payload: List[dict]
@@ -198,12 +185,16 @@ class IngestToSQLite(IIngesterPlugin):
     @staticmethod
     def __create_table(cur: Cursor, destination_table: str, columns: Dict[str, str]):
         """
-        Creates table from give list of columns and table name
+        Creates table from given list of columns and table name
         """
-        columns_as_sql_expression = ",".join(
-            [f"{col_name} {col_type}" for col_name, col_type in columns.items()]
-        )
-        sql = f"CREATE TABLE IF NOT EXISTS {destination_table} ( {columns_as_sql_expression} )"
+        names = [
+            f"{col_name} {col_type}"
+            if " " not in col_name
+            else f'"{col_name}" {col_type}'
+            for col_name, col_type in columns.items()
+        ]
+        columns_as_sql_expression = ",".join(names)
+        sql = f"CREATE TABLE IF NOT EXISTS {destination_table} ( {columns_as_sql_expression} );"
         log.debug(f"Create table using {sql}")
         cur.execute(sql)
 
